@@ -90,6 +90,11 @@ the the 'C' pieces for F<dnsbls>
   test routine should issue SIGINT 
   to child and catch resulting text
 
+=item * t_set_parent(val);
+
+  set the value of "parent"
+  return the previous value
+
 =item * $pid = t_pidrun()
 
   input:	none
